@@ -19,7 +19,11 @@ def async_aiohttp_get_all(urls):
         async with aiohttp.ClientSession() as session:
             async def fetch(url):
                 async with session.get(url) as response:
-                    return await response.json()
+                    try:
+                        return await response.json()
+                    except Exception as exc:
+                        print(exc)
+                        return
             return await asyncio.gather(*[
                 fetch(url) for url in _urls
             ])
@@ -36,7 +40,12 @@ def http_get_all(urls):
 
 def get_query_urls(token, query, engine='keyword', freq='-3h',
                    start_date=None, end_date=None,
-                   roll_period='7d'):
+                   roll_period='7d',
+                   bband_period='21d',
+                   bband_std=2,
+                   ta_indicator='coverage',
+                   z_score=False):
+
     if start_date is not None:
         start_date = quote_url(pd.to_datetime(start_date).isoformat())
     if end_date is not None:
@@ -49,6 +58,9 @@ def get_query_urls(token, query, engine='keyword', freq='-3h',
     elif engine == 'topic':
         host = AdaApiConfig.HOST_TOPIC
         api_path = f"{engine}/{engine}"
+    elif engine == 'news':
+        host = AdaApiConfig.HOST_TOPIC
+        api_path = f"topic/rank-news"
     else:
         raise NotImplemented(f"engine={engine} not supported")
 
@@ -58,14 +70,25 @@ def get_query_urls(token, query, engine='keyword', freq='-3h',
         url_request += f'&start_date={start_date}'
         if end_date is not None:
             url_request += f'&end_date={end_date}'
+
+    if bband_period is not None:
+        url_request += f'&bband_period={bband_period}&bband_std={bband_std}&ta_indicator={ta_indicator}'
+
+    url_request += f'&z_score={z_score}'
     return url_request
 
 
 def load_frame(queries, engine='topic', freq='-1h', roll_period='7d',
-               start_date=None, end_date=None, run_async=True):
+               start_date=None, end_date=None, run_async=True,
+               bband_period=None, bband_std=2, ta_indicator='coverage', z_score=False):
     """
     Query ADASE API to a frame
-    :param run_async:
+    :param z_score: bool, data normalisation
+    :param ta_indicator: str, feature name to apply technical (chart) analysis
+    :param bband_period: str, supported
+        `7d`, `14d`, `28d`, `92d`, `365d`
+    :param bband_std: float, standard deviation
+    :param run_async: bool
     :param queries:  str, syntax varies by engine
         engine='keyword':
             `(+Bitcoin -Luna) OR (+ETH), (+crypto)`
@@ -82,20 +105,28 @@ def load_frame(queries, engine='topic', freq='-1h', roll_period='7d',
     :return: pd.DataFrame
     """
     auth_resp = auth(AdaApiConfig.USERNAME, AdaApiConfig.PASSWORD)
+    queries_split = queries.split(',')
     frames = []
     urls = filter(None, [get_query_urls(auth_resp['access_token'], query, engine=engine, freq=freq,
-                                        start_date=start_date, end_date=end_date,
-                                        roll_period=roll_period)
-                         for query in queries.split(',')])
+                                        start_date=start_date, end_date=end_date, roll_period=roll_period,
+                                        bband_period=bband_period, bband_std=bband_std, z_score=z_score,
+                                        ta_indicator=ta_indicator)
+                         for query in queries_split])
 
     if run_async:
         responses = async_aiohttp_get_all(urls)
     else:
         responses = http_get_all(urls)
 
-    for response in responses:
+    for query, response in zip(queries_split, responses):
         frame = pd.DataFrame(response['data'])
         frame.date_time = pd.DatetimeIndex(frame.date_time.apply(
             lambda dt: datetime.strptime(dt, "%Y%m%d%H")))
-        frames += [frame.set_index(['date_time', 'query', 'source']).unstack(1)]
+        if 'query' not in frame.columns:
+            frames += [frame.assign(**{'query': query})]
+        else:
+            frames += [frame.set_index(['date_time', 'query', 'source']).unstack(1)]
+
+    if engine == 'news':
+        return pd.concat(frames)
     return reduce(lambda l, r: l.join(r, how='outer'), frames).stack(0)
